package config

import (
	"log"
	"os"
	"path/filepath"
	"strings"

	"code.vereign.com/code/viam-apis/authentication"
	"github.com/spf13/viper"
)

import _ "github.com/spf13/viper/remote"

var SystemAuth = &authentication.Authentication{
	Uuid:    "undefined",
	Session: "undefined",
}

var CertificationMethod string
var P PEMReader
var CertificatePEM []byte
var PrivateKeyPEM []byte
var CaCertificatePEM []byte
var VereignCaCertificatePEM []byte
var VereignCaKeyPEM []byte

var MaxMessageSize int

var GrpcListenAddress string
var RestListenAddress string
var DataStorageUrl string
var CertDir string

func SetConfigValues(configFile, etcdURL string) {
	// Set Default Values For Config Variables

	// Vereign API Related
	viper.SetDefault("grpcListenAddress", "localhost:7877")
	viper.SetDefault("restListenAddress", "localhost:7878")
	viper.SetDefault("dataStorageUrl", "localhost:7777")

	viper.SetDefault("viamUUID", "viam-system")
	viper.SetDefault("viamSession", "viam-session")

	viper.SetDefault("maxMessageSize", 64)

	// Certification Related
	// File System Defaults
	viper.SetDefault("certificationMethod", "1")
	viper.SetDefault("certificationURL", ".")
	viper.SetDefault("certificationToken", ".")
	viper.SetDefault("certificationPath", "cert")
	viper.SetDefault("certificationCertFile", "server.crt")
	viper.SetDefault("certificationKeyFile", "server.key")
	viper.SetDefault("certificationCaCertFile", "ca.crt")
	viper.SetDefault("certificationVereignCertFile", "vereign_ca.cer")
	viper.SetDefault("certificationVereignKeyFile", "vereign_ca.key")

	/*
			// Vault Defaults
			viper.SetDefault("certificationMethod", "2")
			viper.SetDefault("certificationURL", "http://10.6.10.119:8200")
			viper.SetDefault("certificationToken", "")
			viper.SetDefault("certificationPath", "/developers/data/devteam/cert")
		    viper.SetDefault("certificationCertFile", "certificateKey")
			viper.SetDefault("certificationKeyFile", "privateKey")
			viper.SetDefault("certificationCaCertFile", "caCertificateKey")
			viper.SetDefault("certificationVereignCertFile", "vereignCaCertificateKey")
			viper.SetDefault("certificationVereignKeyFile", "vereignCaPrivateKey")
	*/

	// Read Config File
	if configFile != "" {
		configName := strings.Split(filepath.Base(configFile), ".")[0]
		configDir := filepath.Dir(configFile)
		viper.SetConfigName(configName)
		viper.AddConfigPath(configDir)
		if err := viper.ReadInConfig(); err != nil {
			log.Printf("can't read config: %v, will use default values", err)
		}
	} else {
		log.Printf("requesting config at "+etcdURL, "/"+os.Getenv("ENV_NAME")+"/"+os.Getenv("CI_PROJECT_NAME")+".json")
		viper.AddRemoteProvider("etcd", etcdURL, "/"+os.Getenv("ENV_NAME")+"/"+os.Getenv("CI_PROJECT_NAME")+".json")
		viper.SetConfigType("json")

		if err := viper.ReadRemoteConfig(); err != nil {
			log.Printf("can't read config: %v, will use default values", err)
		}
	}

	CertificationMethod = viper.GetString("certificationMethod")
	if CertificationMethod == "1" {
		// Read From File System
		P = FilePEMReader{certificationURL: viper.GetString("certificationURL"),
			certificationToken:           viper.GetString("certificationToken"),
			certificationPath:            viper.GetString("certificationPath"),
			certificationCertFile:        viper.GetString("certificationCertFile"),
			certificationKeyFile:         viper.GetString("certificationKeyFile"),
			certificationCaCertFile:      viper.GetString("certificationCaCertFile"),
			certificationVereignCertFile: viper.GetString("certificationVereignCertFile"),
			certificationVereignKeyFile:  viper.GetString("certificationVereignKeyFile")}
	} else if CertificationMethod == "2" {
		// Read From Vault
		P = VaultPEMReader{certificationURL: viper.GetString("certificationURL"),
			certificationToken:           viper.GetString("certificationToken"),
			certificationPath:            viper.GetString("certificationPath"),
			certificationCertFile:        viper.GetString("certificationCertFile"),
			certificationKeyFile:         viper.GetString("certificationKeyFile"),
			certificationCaCertFile:      viper.GetString("certificationCaCertFile"),
			certificationVereignCertFile: viper.GetString("certificationVereignCertFile"),
			certificationVereignKeyFile:  viper.GetString("certificationVereignKeyFile")}
	}

	// Print all config values to log file
	log.Printf("All Settings From Config:")
	as := viper.AllSettings()
	for key, _ := range as {
		log.Printf("%s => %s", key, viper.GetString(key))
	}

	GrpcListenAddress = viper.GetString("grpcListenAddress")
	RestListenAddress = viper.GetString("restListenAddress")

	DataStorageUrl = viper.GetString("dataStorageUrl")

	SystemAuth.Uuid = viper.GetString("viamUUID")
	SystemAuth.Session = viper.GetString("viamSession")

	MaxMessageSize = viper.GetInt("maxMessageSize")

	CertificatePEM = GetCertificatePEM()
	PrivateKeyPEM = GetPrivateKeyPEM()
	CaCertificatePEM = GetCaCertificatePEM()
	VereignCaCertificatePEM = GetVereignCaCertificatePEM()
	VereignCaKeyPEM = GetVereignCaKeyPEM()
}

func GetCertificatePEM() []byte {
	return P.readCertificatePEM()
}

func GetPrivateKeyPEM() []byte {
	return P.readPrivateKeyPEM()
}

func GetCaCertificatePEM() []byte {
	return P.readCaCertificatePEM()
}

func GetVereignCaCertificatePEM() []byte {
	return P.readVereignCaCertificatePEM()
}

func GetVereignCaKeyPEM() []byte {
	return P.readVereignCaKeyPEM()
}
